@@ -1,18 +1,16 @@
 # Fit function per group:
 maxLikEstimator_Gauss_group <- function(S,kappa,means,mu,sigma,...){
-  
-  # nvar <- ncol(kappa) 
-  res <-  sum(diag(S %*% kappa)) + t(means - mu) %*% kappa %*% (means - mu)  - attr(kappa, "logdet")
+
+    # nvar <- ncol(kappa) 
+  res <-  sum(diag(S %*% kappa)) + t(means - mu) %*% kappa %*% (means - mu)  - log(det(kappa))
   
   as.numeric(res)
 }
 
 # Fit function for Gauss ML: -2n* log likelihood
-maxLikEstimator_Gauss <- function(x, model){
-  # Prepare
-  prep <- prepareModel(x, model)
+maxLikEstimator_Gauss <- function(prep){
 
-  # Fit function per group:
+    # Fit function per group:
   fit_per_group <- prep$nPerGroup / prep$nTotal * sapply(prep$groupModels,do.call,what=maxLikEstimator_Gauss_group)
 
   # Sum and return:
