@@ -41,13 +41,13 @@ lvm <- function(
   equal = "none", # Can also be any of the matrices
   baseline_saturated = TRUE, # Leave to TRUE! Only used to stop recursive calls
   estimator = "ML",
-  optimizer = "default",
+  optimizer,
   storedata = FALSE,
-  WLS.V,
+  WLS.W,
   covtype = c("choose","ML","UB"),
   standardize = c("none","z","quantile"),
   sampleStats,
-  verbose=TRUE
+  verbose=FALSE
 ){
   rawts = FALSE
   if (rawts){
@@ -62,8 +62,8 @@ lvm <- function(
   identification <- match.arg(identification)
 
   # WLS weights:
-  if (missing(WLS.V)){
-    WLS.V <- ifelse(!estimator %in% c("WLS","ULS","DWLS"), "none",
+  if (missing(WLS.W)){
+    WLS.W <- ifelse(!estimator %in% c("WLS","ULS","DWLS"), "none",
                     switch(estimator,
                            "WLS" = "full",
                            "ULS" = "identity",
@@ -84,7 +84,7 @@ lvm <- function(
                                fimldata = estimator == "FIML",
                                storedata = storedata,
                                covtype=covtype,
-                               weightsmatrix = WLS.V,
+                               weightsmatrix = WLS.W,
                                verbose=verbose,
                                standardize=standardize)
   }
@@ -96,8 +96,9 @@ lvm <- function(
   # Generate model object:
   model <- generate_psychonetrics(model = "lvm",sample = sampleStats,computed = FALSE, 
                                   equal = equal,identification=identification,
-                                  optimizer = optimizer, estimator = estimator, distribution = "Gaussian",
-                                  rawts = rawts, types = list(latent = latent, residual = residual))
+                                  optimizer =  "nlminb", estimator = estimator, distribution = "Gaussian",
+                                  rawts = rawts, types = list(latent = latent, residual = residual),
+                                  verbose=verbose)
   
   # Submodel:
   latentCov <- latent %in% c("cov","chol")
@@ -153,7 +154,9 @@ lvm <- function(
                                     expmeans = lapply(seq_len(nGroup),function(x)rep(0,nLatent)), sampletable = sampleStats, name = "nu_eta")
   
    # Setup lambda:
-  modMatrices$lambda <- matrixsetup_lambda(lambda, expcov=model@sample@covs, nGroup = nGroup, observednames = sampleStats@variables$label, latentnames = latents, sampletable = sampleStats)
+  modMatrices$lambda <- matrixsetup_lambda(lambda, expcov=model@sample@covs, nGroup = nGroup, 
+                                           observednames = sampleStats@variables$label, latentnames = latents, 
+                                           sampletable = sampleStats, identification = identification)
   
   # Setup beta:
   modMatrices$beta <- matrixsetup_beta(beta, nNode = nLatent, nGroup = nGroup, labels = latents, sampletable = sampleStats)
@@ -164,49 +167,67 @@ lvm <- function(
   
   # For each group:
   for (g in 1:nGroup){
-    # Current cov estimate:
-    curcov <- as.matrix(sampleStats@covs[[g]])
- 
-    # Cur loadings:
-    curLambda <- modMatrices$lambda$start[,,g]
-    if (!is.matrix(curLambda)){
-      curLambda <- as.matrix(curLambda)
-    }
+    expResidSigma[[g]] <- modMatrices$lambda$sigma_epsilon_start[,,g]
+    expLatSigma[[g]] <-  modMatrices$lambda$sigma_zeta_start[,,g]
     
-    # Residual variances, let's start by putting the vars on 1/4 times the observed variances:
-    Theta <- diag(diag(curcov)/4)
-    
-    # Check if this is positive definite:
-    ev <- eigen(curcov - Theta)$values
-    
-    # Shrink until it is positive definite:
-    loop <- 0
-    repeat{
-      ev <- eigen(curcov - Theta)$values
-      if (loop == 100){
-        # give up...
-        
-        Theta <- diag(nrow(Theta))
-        break
-      }
-      if (all(ev>0)){
-        break
-      }
-      Theta <- Theta * 0.9
-      loop <- loop + 1
-    }
-    
-    # Expected residual sigma:
-    expResidSigma[[g]] <- Theta
-    
-    # This means that the factor-part is expected to be:
-    factorPart <- curcov - Theta
-    
-    # Let's take a pseudoinverse:
-    inv <- corpcor::pseudoinverse(kronecker(curLambda,curLambda))
-    
-    # And obtain psi estimate:
-    expLatSigma[[g]] <- matrix(inv %*% as.vector(factorPart),nLatent,nLatent)
+      
+    # # Current cov estimate:
+    # curcov <- as.matrix(sampleStats@covs[[g]])
+    # 
+    # 
+    # # Cur loadings:
+    # curLambda <- modMatrices$lambda$start[,,g]
+    # if (!is.matrix(curLambda)){
+    #   curLambda <- as.matrix(curLambda)
+    # }
+    # 
+    # ### Run factor start ###:
+    # # facStart <- factorstart(curcov, modMatrices$lambda[[1]][,,1])
+    # 
+    # 
+    # ###
+    # 
+    # 
+    # # Residual variances, let's start by putting the vars on 1/4 times the observed variances:
+    # Theta <- diag(diag(curcov)/4)
+    # # Theta <- modMatrices$lambda$thetaStart[,,g]
+    # fa <- factanal(covmat = curcov, factors = nLatent, covar = TRUE)
+    # 
+    # fa <- fa(r = curcov, nfactors = nLatent, rotate = "promax", covar = TRUE)
+    # 
+    # Theta <- diag(fa$uniquenesses)
+    # 
+    # # Check if this is positive definite:
+    # ev <- eigen(curcov - Theta)$values
+    # 
+    # # Shrink until it is positive definite:
+    # loop <- 0
+    # repeat{
+    #   ev <- eigen(curcov - Theta)$values
+    #   if (loop == 100){
+    #     # give up...
+    #     
+    #     Theta <- diag(nrow(Theta))
+    #     break
+    #   }
+    #   if (all(ev>0)){
+    #     break
+    #   }
+    #   Theta <- Theta * 0.9
+    #   loop <- loop + 1
+    # }
+    # 
+    # # Expected residual sigma:
+    # expResidSigma[[g]] <- Theta
+    # 
+    # # This means that the factor-part is expected to be:
+    # factorPart <- curcov - Theta
+    # 
+    # # Let's take a pseudoinverse:
+    # inv <- corpcor::pseudoinverse(kronecker(curLambda,curLambda))
+    # 
+    # # And obtain psi estimate:
+    # expLatSigma[[g]] <- matrix(inv %*% as.vector(factorPart),nLatent,nLatent)
   }
   
   # Latent varcov:
@@ -246,7 +267,8 @@ lvm <- function(
                                                 nNode = nLatent, 
                                                 nGroup = nGroup, 
                                                 labels = latents,
-                                           equal = "delta_zeta" %in% equal, sampletable = sampleStats) 
+                                           equal = "delta_zeta" %in% equal, sampletable = sampleStats,
+                                           omegaStart =  modMatrices$omega_zeta$start) 
   } else if (latent == "prec"){
     
     # Add omega matrix:
@@ -294,7 +316,8 @@ lvm <- function(
                                                 nNode = nNode, 
                                                 nGroup = nGroup, 
                                                 labels = sampleStats@variables$label,
-                                                equal = "delta_epsilon" %in% equal, sampletable = sampleStats) 
+                                                equal = "delta_epsilon" %in% equal, sampletable = sampleStats,
+                                                omegaStart =  modMatrices$omega_epsilon$start) 
   } else if (residual == "prec"){
     
     # Add omega matrix:
@@ -326,11 +349,11 @@ lvm <- function(
     L_eta = psychonetrics::eliminationMatrix(nLatent), # Elinimation matrix
     Dstar = psychonetrics::duplicationMatrix(nNode,diag = FALSE), # Strict duplicaton matrix
     Dstar_eta = psychonetrics::duplicationMatrix(nLatent,diag = FALSE), # Strict duplicaton matrix
-    In = Diagonal(nNode), # Identity of dim n
-    Inlatent = Diagonal(nLatent),
-    C = as(lavaan::lav_matrix_commutation(nNode, nLatent),"sparseMatrix"),
-    Cbeta = as(lavaan::lav_matrix_commutation(nLatent, nLatent),"sparseMatrix"),
-    C_chol = as(lavaan::lav_matrix_commutation(nNode, nNode),"sparseMatrix"),
+    In = as(diag(nNode),"dgCMatrix"), # Identity of dim n
+    Inlatent = as(diag(nLatent),"dgCMatrix"),
+    C = as(lavaan::lav_matrix_commutation(nNode, nLatent),"dgCMatrix"),
+    Cbeta = as(lavaan::lav_matrix_commutation(nLatent, nLatent),"dgCMatrix"),
+    C_chol = as(lavaan::lav_matrix_commutation(nNode, nNode),"dgCMatrix"),
     A = psychonetrics::diagonalizationMatrix(nNode),
     Aeta = psychonetrics::diagonalizationMatrix(nLatent)
   )
@@ -389,6 +412,12 @@ lvm <- function(
     model <- identify(model)
   }
   
+  if (missing(optimizer)){
+    model <- setoptimizer(model, "default")
+  } else {
+    model <- setoptimizer(model, optimizer)
+  }
+  
   # Return model:
   return(model)
 }
\ No newline at end of file
