@@ -51,17 +51,18 @@ tsdlvm1 <- function(
   baseline_saturated = TRUE, # Leave to TRUE! Only used to stop recursive calls
   # fitfunctions, # Leave empty
   estimator = "ML",
-  optimizer = "default", # ucminf
+  optimizer,
   storedata = FALSE,
   sampleStats,
   covtype = c("choose","ML","UB"),
   centerWithin = FALSE,
-  standardize = c("none","z","quantile")
+  standardize = c("none","z","quantile"),
+  verbose = FALSE
 ){
   contemporaneous <- match.arg(contemporaneous)
   residual <- match.arg(residual)
   identification <- match.arg(identification)
-
+  
   # FIXME: Not sure why needed...
   if (missing(vars)) vars2 <- NULL else vars2 <- vars
   if (missing(idvar)) idvar <- NULL
@@ -100,14 +101,15 @@ tsdlvm1 <- function(
                                storedata = storedata,
                                covtype=covtype,
                                standardize=standardize,
+                               verbose=verbose,
                                weightsmatrix = ifelse(!estimator %in% c("WLS","ULS","DWLS"), "none",
                                                       switch(estimator,
-                                                        "WLS" = "full",
-                                                        "ULS" = "identity",
-                                                        "DWLS" = "diag"
+                                                             "WLS" = "full",
+                                                             "ULS" = "identity",
+                                                             "DWLS" = "diag"
                                                       )))    
   }
-
+  
   
   # Check if number of variables is an even number:
   if ( nrow(sampleStats@variables) %% 2 != 0){
@@ -141,7 +143,8 @@ tsdlvm1 <- function(
                                                epsilon = residual),
                                   sample = sampleStats,computed = FALSE, 
                                   equal = equal,
-                                  optimizer = optimizer, estimator = estimator, distribution = "Gaussian")
+                                  optimizer =  "nlminb", estimator = estimator, distribution = "Gaussian",
+                                  verbose=verbose)
   
   # Number of groups:
   nGroup <- nrow(model@sample@groups)
@@ -161,12 +164,12 @@ tsdlvm1 <- function(
   
   # Fix exo means
   modMatrices$exo_means <- matrixsetup_mu(, nNode = nNode, nGroup = nGroup, labels = varnames, equal = "exo_means" %in% equal,
-                expmeans = expMeans1, sampletable = sampleStats, name = "exo_means")
+                                          expmeans = expMeans1, sampletable = sampleStats, name = "exo_means")
   
   
   # Fix nu
   modMatrices$nu <- matrixsetup_mu(nu, nNode = nNode, nGroup = nGroup, labels = varnames,equal = "nu" %in% equal,
-                                    expmeans = expMeans2, sampletable = sampleStats, name = "nu")
+                                   expmeans = expMeans2, sampletable = sampleStats, name = "nu")
   
   # Fix between-subject means:
   modMatrices$mu_eta <- matrixsetup_mu(mu_eta,nNode = nLat, nGroup = nGroup, labels = latents, equal = "mu_eta" %in% equal,
@@ -199,16 +202,19 @@ tsdlvm1 <- function(
   modMatrices$lambda <- matrixsetup_lambda(lambda, expcov=S0est, nGroup = nGroup, observednames = varnames, latentnames = latents,
                                            sampletable = sampleStats, name = "lambda", equal = "lambda" %in% equal)
   
-
+  
   # Quick and dirty sigma_zeta estimate:
-  prior_sig_zeta <- lapply(seq_along(S0est),function(i){
-    # Let's take a pseudoinverse:
-    curLam <- Matrix(as.vector(modMatrices$lambda$start[,,i,drop=FALSE]),nNode,nLat)
-    
-    inv <- corpcor::pseudoinverse(as.matrix(kronecker(curLam,curLam)))
-    
-    # And obtain psi estimate:
-    matrix(inv %*% as.vector(S0est[[i]])/2,nLat,nLat)
+  # prior_sig_zeta <- lapply(seq_along(S0est),function(i){
+  #   # Let's take a pseudoinverse:
+  #   curLam <- matrix(as.vector(modMatrices$lambda$start[,,i,drop=FALSE]),nNode,nLat)
+  # 
+  #   inv <- corpcor::pseudoinverse(as.matrix(kronecker(curLam,curLam)))
+  # 
+  #   # And obtain psi estimate:
+  #   matrix(inv %*% as.vector(S0est[[i]])/2,nLat,nLat)
+  # })
+  prior_sig_zeta <- lapply(seq_len(nGroup),function(g){
+    modMatrices$lambda$sigma_zeta_start[,,g]
   })
   
   # Setup latent varcov:
@@ -233,6 +239,10 @@ tsdlvm1 <- function(
                                        equal = "beta" %in% equal, sampletable = sampleStats)
   
   
+  prior_sig_epsilon <- lapply(seq_len(nGroup),function(g){
+    modMatrices$lambda$sigma_epsilon_start[,,g]
+  })
+  
   # Setup residuals:
   modMatrices <- c(modMatrices,
                    matrixsetup_flexcov(sigma_epsilon,lowertri_epsilon,omega_epsilon,delta_epsilon,kappa_epsilon,
@@ -241,7 +251,7 @@ tsdlvm1 <- function(
                                        sampleStats= sampleStats,
                                        equal = equal,
                                        nNode = nNode,
-                                       expCov = lapply(1:nGroup,function(x)diag(0.5,nNode)),
+                                       expCov = prior_sig_epsilon,
                                        nGroup = nGroup,
                                        labels = varnames
                    ))
@@ -253,19 +263,19 @@ tsdlvm1 <- function(
   # Store in model:
   model@parameters <- pars$partable
   model@matrices <- pars$mattable
-# 
-#   model@extramatrices <- list(
-#     D =  psychonetrics::duplicationMatrix(nNode*2), # Toeplitz matrix D 
-#     D2 = psychonetrics::duplicationMatrix(nNode), # non-strict duplciation matrix
-#     L = psychonetrics::eliminationMatrix(nNode), # Elinimation matrix
-#     Dstar = psychonetrics::duplicationMatrix(nNode,diag = FALSE), # Strict duplicaton matrix
-#     In = Diagonal(nNode), # Identity of dim n
-#     In2 = Diagonal(nNode), # Identity of dim n^2
-#     A = psychonetrics::diagonalizationMatrix(nNode),
-#     C = as(lavaan::lav_matrix_commutation(nNode,nNode),"sparseMatrix")
-#     # P=P # Permutation matrix
-#   )
-#   
+  # 
+  #   model@extramatrices <- list(
+  #     D =  psychonetrics::duplicationMatrix(nNode*2), # Toeplitz matrix D 
+  #     D2 = psychonetrics::duplicationMatrix(nNode), # non-strict duplciation matrix
+  #     L = psychonetrics::eliminationMatrix(nNode), # Elinimation matrix
+  #     Dstar = psychonetrics::duplicationMatrix(nNode,diag = FALSE), # Strict duplicaton matrix
+  #     In = Diagonal(nNode), # Identity of dim n
+  #     In2 = Diagonal(nNode), # Identity of dim n^2
+  #     A = psychonetrics::diagonalizationMatrix(nNode),
+  #     C = as(lavaan::lav_matrix_commutation(nNode,nNode),"sparseMatrix")
+  #     # P=P # Permutation matrix
+  #   )
+  #   
   
   model@extramatrices <- list(
     # Entire duplication matrix needed for likelihood:
@@ -284,8 +294,8 @@ tsdlvm1 <- function(
     # Dstar_between = psychonetrics::duplicationMatrix(nLat,diag = FALSE),  
     
     # Identity matrices:
-    I_y = Diagonal(nNode),
-    I_eta = Diagonal(nLat),
+    I_y = as(diag(nNode),"dgCMatrix"),
+    I_eta = as(diag(nLat),"dgCMatrix"),
     # I_within = Diagonal(nLat),
     # I_between = Diagonal(nLat),
     
@@ -296,9 +306,9 @@ tsdlvm1 <- function(
     # A_between = psychonetrics::diagonalizationMatrix(nLat),
     
     # Commutation matrices:
-    C_y_y = as(lavaan::lav_matrix_commutation(nNode, nNode),"sparseMatrix"),
-    C_y_eta = as(lavaan::lav_matrix_commutation(nNode, nLat),"sparseMatrix"),
-    C_eta_eta = as(lavaan::lav_matrix_commutation(nLat, nLat),"sparseMatrix"),
+    C_y_y = as(lavaan::lav_matrix_commutation(nNode, nNode),"dgCMatrix"),
+    C_y_eta = as(lavaan::lav_matrix_commutation(nNode, nLat),"dgCMatrix"),
+    C_eta_eta = as(lavaan::lav_matrix_commutation(nLat, nLat),"dgCMatrix"),
     
     # 
     # C_y_within = as(lavaan::lav_matrix_commutation(nVar, nLat),"sparseMatrix"),
@@ -311,19 +321,19 @@ tsdlvm1 <- function(
     L_eta = psychonetrics::eliminationMatrix(nLat)
   )
   
-    # Come up with P...
-    # Dummy matrix to contain indices:
-    dummySigma <- matrix(0,nNode*2,nNode*2)
-    smallMat <- matrix(0,nNode,nNode)
-    dummySigma[1:nNode,1:nNode][lower.tri(smallMat,diag=TRUE)] <- seq_len(nNode*(nNode+1)/2)
-    dummySigma[nNode + (1:nNode),nNode + (1:nNode)][lower.tri(smallMat,diag=TRUE)] <- max(dummySigma) + seq_len(nNode*(nNode+1)/2)
-    dummySigma[nNode + (1:nNode),1:nNode] <- max(dummySigma) + seq_len(nNode^2)
-    inds <- dummySigma[lower.tri(dummySigma,diag=TRUE)]
-    
-    # P matrix:
-    # P <- bdiag(Diagonal(nNode*2),sparseMatrix(j=seq_along(inds),i=inds))
-    model@extramatrices$P <- bdiag(Diagonal(nNode*2),sparseMatrix(j=seq_along(inds),i=order(inds)))
-
+  # Come up with P...
+  # Dummy matrix to contain indices:
+  dummySigma <- matrix(0,nNode*2,nNode*2)
+  smallMat <- matrix(0,nNode,nNode)
+  dummySigma[1:nNode,1:nNode][lower.tri(smallMat,diag=TRUE)] <- seq_len(nNode*(nNode+1)/2)
+  dummySigma[nNode + (1:nNode),nNode + (1:nNode)][lower.tri(smallMat,diag=TRUE)] <- max(dummySigma) + seq_len(nNode*(nNode+1)/2)
+  dummySigma[nNode + (1:nNode),1:nNode] <- max(dummySigma) + seq_len(nNode^2)
+  inds <- dummySigma[lower.tri(dummySigma,diag=TRUE)]
+  
+  # P matrix:
+  # P <- bdiag(Diagonal(nNode*2),sparseMatrix(j=seq_along(inds),i=inds))
+  model@extramatrices$P <- bdiag(Diagonal(nNode*2),sparseMatrix(j=seq_along(inds),i=order(inds)))
+  model@extramatrices$P  <- as(as.matrix(model@extramatrices$P), "dgCMatrix")
   
   # Form the model matrices
   model@modelmatrices <- formModelMatrices(model)
@@ -380,6 +390,12 @@ tsdlvm1 <- function(
     model <- identify(model)
   }
   
+  if (missing(optimizer)){
+    model <- setoptimizer(model, "default")
+  } else {
+    model <- setoptimizer(model, optimizer)
+  }
+  
   # Return model:
   return(model)
 }
